"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
const tslib_1 = require("tslib");
const log_1 = require("@dash4/log");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const execa_1 = tslib_1.__importDefault(require("execa"));
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const path_1 = tslib_1.__importDefault(require("path"));
const read_pkg_1 = tslib_1.__importDefault(require("read-pkg"));
const write_pkg_1 = tslib_1.__importDefault(require("write-pkg"));
const Config_1 = require("./Config/Config");
const get_lerna_packages_1 = require("./get-lerna-packages");
const get_readme_config_1 = require("./get-readme-config");
const get_test_config_1 = require("./get-test-config");
const utils_1 = require("./utils");
function asyncForEach(array, onIt) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        for (let index = 0; index < array.length; index++) {
            // eslint-disable-next-line
            yield onIt(array[index], index, array);
        }
    });
}
function collectTab({ cwd, packagePath, lerna, parentUsesYarn }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        let packages = {};
        let configs = [];
        const packageData = yield read_pkg_1.default({ cwd, normalize: false });
        const usesYarn = parentUsesYarn ||
            (yield utils_1.hasFile(path_1.default.join(cwd, 'yarn.lock'))) ||
            (utils_1.getScript(packageData, 'start') || '').includes('yarn');
        const command = usesYarn ? 'yarn' : 'npm run';
        function append(results) {
            packages = Object.assign(Object.assign({}, packages), results.packages);
            configs = configs.concat(results.configs);
        }
        const npmScriptsCmds = [
            utils_1.hasScript(packageData, 'bootstrap') ? `${command} bootstrap` : undefined,
            utils_1.hasScript(packageData, 'build') ? `${command} build` : undefined,
            utils_1.hasScript(packageData, 'watch') ? `${command} watch` : undefined,
            utils_1.hasScript(packageData, 'test') ? `${command} test` : undefined,
            utils_1.hasScript(packageData, 'lint') ? `${command} lint` : undefined,
            utils_1.hasScript(packageData, 'clean') ? `${command} clean` : undefined,
            utils_1.hasScript(packageData, 'flow') ? `${command} flow` : undefined,
            utils_1.hasScript(packageData, 'compile') ? `${command} compile` : undefined,
            utils_1.hasScript(packageData, 'format') ? `${command} format` : undefined,
            utils_1.hasScript(packageData, 'prettier') ? `${command} prettier` : undefined,
            utils_1.hasScript(packageData, 'coverage') ? `${command} coverage` : undefined,
            utils_1.hasScript(packageData, 'prepare') ? `${command} prepare` : undefined,
            utils_1.hasScript(packageData, 'validate') ? `${command} validate` : undefined,
            utils_1.hasScript(packageData, 'deploy') ? `${command} deploy` : undefined,
            utils_1.hasScript(packageData, 'docs') ? `${command} docs` : undefined,
            utils_1.hasScript(packageData, 'build-storybook') ? `${command} build-storybook` : undefined,
        ].filter((cmd) => cmd);
        if (!lerna) {
            configs.push({
                pluginName: 'PluginDependencies',
            });
            packages['@dash4/plugin-dependencies'] = true;
        }
        if (lerna && !packagePath) {
            configs.push({
                pluginName: 'PluginDependencies',
                options: {
                    lerna,
                    installProcess: {
                        title: 'run bootstrap',
                        cmd: `${command} bootstrap`,
                        cwd: '/',
                    },
                },
            });
            packages['@dash4/plugin-dependencies'] = true;
        }
        if (utils_1.hasScript(packageData, 'start')) {
            configs.push({
                pluginName: 'PluginTerminal',
                options: {
                    cmd: `${command.replace(' run', '')} start`,
                    cwd: packagePath,
                    autostart: true,
                },
            });
            packages['@dash4/plugin-terminal'] = true;
        }
        if (utils_1.hasScript(packageData, 'storybook')) {
            configs.push({
                pluginName: 'PluginTerminal',
                options: {
                    cmd: `${command} storybook`,
                    cwd: packagePath,
                },
            });
            packages['@dash4/plugin-terminal'] = true;
        }
        append(yield get_readme_config_1.getReadmeConfig({
            packagePath,
            cwd,
        }));
        append(yield get_test_config_1.getTestConfig({
            packagePath,
            packageData,
            usesYarn,
        }));
        if (npmScriptsCmds.length > 0) {
            configs.push({
                pluginName: 'PluginNpmScripts',
                options: {
                    scripts: npmScriptsCmds.map((cmd) => ({
                        title: cmd ? cmd.replace(/^npm run /, '').replace(/^yarn /, '') : '',
                        cmd,
                        cwd: packagePath,
                    })),
                },
            });
            packages['@dash4/plugin-npm-scripts'] = true;
        }
        return { packages, configs };
    });
}
function init(cwd, options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const spin = log_1.spinner('cli', 'Collect required changes');
        spin.start();
        if (!(yield utils_1.hasFile(cwd, 'package.json'))) {
            spin.fail('package.json not found!');
            process.kill(1);
            return;
        }
        if ((yield utils_1.hasFile(options.config)) && !options.force) {
            spin.fail('Dash4 is already installed!');
            process.kill(1);
            return;
        }
        const config = new Config_1.Config({ port: options.port });
        let packages = { '@dash4/server': true };
        const packageData = yield read_pkg_1.default({ cwd, normalize: false });
        const isLerna = yield utils_1.hasFile(cwd, 'lerna.json');
        const usesYarn = (yield utils_1.hasFile(path_1.default.join(cwd, 'yarn.lock'))) || (utils_1.getScript(packageData, 'start') || '').includes('yarn');
        const collection = yield collectTab({
            cwd,
            lerna: isLerna ? path_1.default.join('lerna.json') : undefined,
        });
        packages = Object.assign(Object.assign({}, packages), collection.packages);
        collection.configs.forEach((_config) => {
            config.addPlugin('Root', _config.pluginName, _config.options);
        });
        // lerna
        if (isLerna) {
            spin.text('collect changes for lerna monorepository');
            const lernaPackages = yield get_lerna_packages_1.getLernaPackages(cwd);
            asyncForEach(lernaPackages, (lernaPackage) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                try {
                    const tabName = (yield fs_extra_1.default.readJSON(path_1.default.join(cwd, lernaPackage, 'package.json'))).name ||
                        path_1.default.basename(lernaPackage);
                    const _collection = yield collectTab({
                        cwd: path_1.default.join(cwd, lernaPackage),
                        packagePath: lernaPackage,
                        lerna: path_1.default.join('lerna.json'),
                        parentUsesYarn: usesYarn,
                    });
                    packages = Object.assign(Object.assign({}, packages), _collection.packages);
                    _collection.configs.forEach((_config) => {
                        config.addPlugin(tabName, _config.pluginName, _config.options);
                    });
                }
                catch (err) {
                    return;
                }
            }));
        }
        // add npm scripts
        spin.text('add dash4 script');
        packageData.scripts = packageData.scripts || {};
        packageData.scripts.dash4 = 'dash4';
        yield write_pkg_1.default(path_1.default.join(cwd, 'package.json'), packageData);
        // write dash.config
        spin.text('create dash4 configuration');
        yield fs_extra_1.default.writeFile(options.config, config.toString());
        // install dependencies
        spin.text('install dependencies');
        try {
            if (usesYarn) {
                spin.text(`yarn add -D -W ${Object.keys(packages).join(' ')}`);
                yield execa_1.default('yarn', ['add', '-D', '-W', ...Object.keys(packages)]);
            }
            else {
                spin.text(`npm i -D ${Object.keys(packages).join(' ')}`);
                yield execa_1.default('npm', ['i', '-D', ...Object.keys(packages)]);
            }
        }
        catch (err) {
            spin.fail(`${chalk_1.default.bold('Could not install dependencies:')}\n${err}`);
            process.kill(1);
        }
        spin.succeed(`your Dash4 dashboard is installed and ready. Run "${usesYarn ? 'yarn' : 'npm run'} dash4"`);
    });
}
exports.init = init;
//# sourceMappingURL=index.js.map